<?php
/******************************************************************************/
//                                                                            //
//                           InstantCMS v1.10.6                               //
//                        http://www.instantcms.ru/                           //
//                                                                            //
//                   written by InstantCMS Team, 2007-2015                    //
//                produced by InstantSoft, (www.instantsoft.ru)               //
//                                                                            //
//                        LICENSED BY GNU/GPL v2                              //
//                                                                            //
/******************************************************************************/

class p_edit_post extends cmsPlugin {

    public $config = array (
        'edit_time'   => 0,
        'delete_time' => 0
    );

    public function __construct(){

        $this->info = array(
            'plugin'      => 'p_edit_post',
            'title'       => 'Ограничение редактирования по времени',
            'description' => 'Задает ограничение на редактирование и удаление поста в блоге, редактирование статей',
            'author'      => '<a href="http://www.instantvideo.ru/" target="_blank">InstantMedia team</a>',
            'version'     => '1.2',
            'published'   => 1
        );

        $this->events = array(
            'EDIT_POST',
            'PRE_EDIT_ARTICLE',
            'DELETE_POST'
        );

        parent::__construct();

    }

    public function execute($event='', $item=array()){

		global $_LANG;

		if(cmsUser::getInstance()->is_admin) { return $item; }

        switch ($event){

            case 'PRE_EDIT_ARTICLE':
            case 'EDIT_POST':

				if(!$this->config['edit_time'] || empty($item['pubdate'])) { return $item; }

				$end_sec = $this->diffTime($item['pubdate'], $this->config['edit_time']);

				if($end_sec<=0){

					cmsCore::addSessionMessage($_LANG['P_EDIT_POST_TIME_END'], 'error');
					cmsCore::redirectBack();

				} else {

					cmsCore::addSessionMessage($_LANG['P_EDIT_POST_TIME'].string_date_age(date("Y-m-d H:i:s", $end_sec+time())), 'info');

				}

            break;

            case 'DELETE_POST':

				if(!$this->config['delete_time']) { return $item; }

				$end_sec = $this->diffTime($item['pubdate'], $this->config['delete_time']);

				if($end_sec<=0){
					cmsCore::addSessionMessage($_LANG['P_EDIT_POST_DEL_TIME_END'], 'error');
					cmsCore::jsonOutput(array('error' => false, 'redirect'  => cmsCore::getBackURL()));
				}

            break;

        }

        return $item;

    }

    private function diffTime($pubdate, $time){

		$now      = time();
		$date     = strtotime($pubdate);
		$diff_sec = $now - $date;
		$end_sec  = $time*60*60 - $diff_sec;

        return $end_sec;

    }

}

function string_date_age($date){

    if (!$date) { return; }

    global $_LANG;

    $diff = real_date_diff($date);

    $diff_str = array();

    if ($diff[2]){
        $diff_str[] = cmsCore::spellCount($diff[2], $_LANG['DAY1'], $_LANG['DAY2'], $_LANG['DAY10']);
    }
    if ($diff[3]){
        $diff_str[] = cmsCore::spellCount($diff[3], $_LANG['HOUR1'], $_LANG['HOUR2'], $_LANG['HOUR10']);
    }
    if ($diff[4]){
        $diff_str[] = cmsCore::spellCount($diff[4], $_LANG['MINUTU1'], $_LANG['MINUTE2'], $_LANG['MINUTE10']);
    }

    if (!$diff_str) {
        return $_LANG['LESS_MINUTE'];
    } else {
        $diff_str = trim( implode(' ', $diff_str) );
        return $diff_str;
    }

}
/**
 * Возвращает разницу между датами в виде массива
 *
 * Возвращает массив, в котором элементы:
 *  0 => число лет
 *  1 => число месяцев
 *  2 => число дней
 *  3 => число часов
 *  4 => число минут
 *  5 => число секунд
 *
 * @author Олег Савватеев @ http://savvateev.org
 *
 * @param string $date1
 * @param string $date2
 * @return array
 */
function real_date_diff($date1, $date2 = NULL){

    $diff = array();

    if (!is_string($date1)){ return false; }

    //Если вторая дата не задана принимаем ее как текущую
    if(!$date2) {
        $cd = getdate();
        $date2 = $cd['year'].'-'.$cd['mon'].'-'.$cd['mday'].' '.$cd['hours'].':'.$cd['minutes'].':'.$cd['seconds'];
    }

    //Преобразуем даты в массив
    $pattern = '/(\d+)-(\d+)-(\d+)(\s+(\d+):(\d+):(\d+))?/';
    preg_match($pattern, $date1, $matches);
    $d1 = array((int)$matches[1], (int)$matches[2], (int)$matches[3], (int)$matches[5], (int)$matches[6], (int)$matches[7]);
    preg_match($pattern, $date2, $matches);
    $d2 = array((int)$matches[1], (int)$matches[2], (int)$matches[3], (int)$matches[5], (int)$matches[6], (int)$matches[7]);

    //Если вторая дата меньше чем первая, меняем их местами
    for($i=0; $i<count($d2); $i++) {
        if($d2[$i]>$d1[$i]) break;
        if($d2[$i]<$d1[$i]) {
            $t = $d1;
            $d1 = $d2;
            $d2 = $t;
            break;
        }
    }

    //Вычисляем разность между датами (как в столбик)
    $md1 = array(31, $d1[0]%4||(!($d1[0]%100)&&$d1[0]%400)?28:29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
    $md2 = array(31, $d2[0]%4||(!($d2[0]%100)&&$d2[0]%400)?28:29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
    $min_v = array(NULL, 1, 1, 0, 0, 0);
    $max_v = array(NULL, 12, $d2[1]==1?$md2[11]:$md2[$d2[1]-2], 23, 59, 59);
    for($i=5; $i>=0; $i--) {
        if($d2[$i]<$min_v[$i]) {
            $d2[$i-1]--;
            $d2[$i]=$max_v[$i];
        }
        $diff[$i] = $d2[$i]-$d1[$i];
        if($diff[$i]<0) {
            $d2[$i-1]--;
            $i==2 ? $diff[$i] += $md1[$d1[1]-1] : $diff[$i] += $max_v[$i]-$min_v[$i]+1;
        }
    }

    //Возвращаем результат
    return $diff;

}
